import * as cbor from "microcbor";
import { assert, signalInvalidType } from "@canvas-js/utils";
import { MessageId } from "./MessageId.js";
export class MessageSet {
    static decode(bytes) {
        const links = cbor.decode(bytes);
        return new MessageSet(links.map(MessageId.decode));
    }
    #map = new Map();
    constructor(entries) {
        for (const messageId of entries ?? []) {
            if (messageId instanceof MessageId) {
                this.add(messageId);
            }
            else if (messageId instanceof Uint8Array) {
                this.add(MessageId.decode(messageId));
            }
            else if (typeof messageId === "string") {
                this.add(MessageId.encode(messageId));
            }
            else {
                signalInvalidType(messageId);
            }
        }
    }
    get size() {
        return this.#map.size;
    }
    has(messageId) {
        return this.#map.has(messageId.id);
    }
    add(messageId) {
        this.#map.set(messageId.id, messageId);
    }
    delete(messageId) {
        this.#map.delete(messageId.id);
    }
    [Symbol.iterator]() {
        return this.#map.values();
    }
    encode() {
        const links = [];
        for (const messageId of this.#map.values()) {
            links.push(messageId.key);
        }
        return cbor.encode(links);
    }
    max() {
        assert(this.#map.size > 0, "empty message set");
        let max = null;
        for (const messageId of this.#map.values()) {
            if (max === null || messageId.id > max.id) {
                max = messageId;
            }
        }
        return max;
    }
    min() {
        assert(this.#map.size > 0, "empty message set");
        let min = null;
        for (const messageId of this.#map.values()) {
            if (min === null || messageId.id < min.id) {
                min = messageId;
            }
        }
        return min;
    }
}
